==> nodejs_interop-0.7.0.tar.gz: lib/src/io/file/file_stat.dart <==
part of nodejs_interop.io;
// ignore_for_file: prefer_constructors_over_static_methods

/// Represents the result of calling the POSIX [`stat`](http://manpages.ubuntu.com/manpages/xenial/man2/stat.2.html) function on a file system entity.
/// It is an immutable object, representing the snapshotted values returned by the `stat()` call.
class FileStat implements f.FileStat {

  /// A reference to the `fs` module.
  static final FSModule _fs = require('fs');

  /// Creates new file stats from the specified native [stats].
  factory FileStat.fromStats(Stats stats) {
    var type = f.FileSystemEntityType.NOT_FOUND;
    if (stats.isDirectory()) type = f.FileSystemEntityType.DIRECTORY;
    else if (stats.isFile()) type = f.FileSystemEntityType.FILE;
    else if (stats.isSymbolicLink()) type = f.FileSystemEntityType.LINK;

    return new FileStat._internal(type,
      accessed: stats.atime,
      changed: stats.ctime,
      mode: stats.mode,
      modified: stats.mtime,
      size: stats.size
    );
  }

  /// Creates new file stats.
  FileStat._internal(this.type, {
    this.accessed,
    this.changed,
    this.mode = 0,
    this.modified,
    this.size = -1
  });

  /// Calls the operating system's `stat()` function on the specified [path].
  ///
  /// Returns a [Future<FileStat>] object containing the data returned by `stat()`.
  /// If the call fails, completes the future with a [FileStat] object with [type] set to [f.FileSystemEntityType.NOT_FOUND] and the other fields invalid.
  static Future<FileStat> stat(String path) async {
    var completer = new Completer<FileStat>();
    _fs.lstat(path, allowInterop((error, [stats]) => completer.complete(
      error != null ? new FileStat._internal(f.FileSystemEntityType.NOT_FOUND) : new FileStat.fromStats(stats)
    )));

    return completer.future;
  }

  /// Synchronously calls the operating system's `stat()` function on the specified [path].
  ///
  /// Returns a [FileStat] object containing the data returned by `stat()`.
  /// If the call fails, returns a [FileStat] object with [type] set to [f.FileSystemEntityType.NOT_FOUND] and the other fields invalid.
  static FileStat statSync(String path) {
    try { return new FileStat.fromStats(_fs.lstatSync(path)); }
    on JSError { return new FileStat._internal(f.FileSystemEntityType.NOT_FOUND); }
  }

  /// The time of the last access to the data of the file system entity.
  /// On Windows platforms, this may have 1 day granularity, and be out of date by an hour.
  @override
  final DateTime accessed;

  /// The time of the last change to the data or metadata of the file system object.
  /// On Windows platforms, this is instead the file creation time.
  @override
  final DateTime changed;

  /// The time of the last change to the data of the file system object.
  @override
  final DateTime modified;

  /// The mode of the file system entity. Permissions are encoded in the lower 16 bits of this number.
  @override
  final int mode;

  /// The total size, in bytes, of the file system entity.
  @override
  final int size;

  /// The type of the file system entity.
  /// If the call to `stat()` fails, the type of the returned entity is [f.FileSystemEntityType.NOT_FOUND].
  @override
  final f.FileSystemEntityType type;

  /// Returns the [mode] value as a human-readable string, in the format `"rwxrwxrwx"`,
  /// reflecting the user, group, and world permissions to read, write, and execute the file system entity,
  /// with `"-"` replacing the letter for missing permissions.
  ///
  /// Extra permission bits may be represented by prepending `"(suid)"`, `"(guid)"`, and/or `"(sticky)"` to the mode string.
  @override
  String modeString() {
    var codes = const ['---', '--x', '-w-', '-wx', 'r--', 'r-x', 'rw-', 'rwx'];
    var permissions = mode & 0xFFF;

    var result = [];
    if ((permissions & 0x800) != 0) result.add('(suid) ');
    if ((permissions & 0x400) != 0) result.add('(guid) ');
    if ((permissions & 0x200) != 0) result.add('(sticky) ');

    result
      ..add(codes[(permissions >> 6) & 0x7])
      ..add(codes[(permissions >> 3) & 0x7])
      ..add(codes[permissions & 0x7]);

    return result.join();
  }
}

==> nodejs_interop-0.8.0.tar.gz: lib/src/io/file/file_stat.dart <==
part of nodejs_interop.io;

/// Represents the result of calling the POSIX [`stat`](http://manpages.ubuntu.com/manpages/xenial/man2/stat.2.html) function on a file system entity.
/// It is an immutable object, representing the snapshotted values returned by the `stat()` call.
class FileStat implements f.FileStat {

  /// A reference to the `fs` module.
  static final FSModule _fs = require('fs');

  /// Creates new file stats from the specified native [stats].
  factory FileStat._fromStats(Stats stats) {
    var type = f.FileSystemEntityType.NOT_FOUND;
    if (stats.isDirectory()) type = f.FileSystemEntityType.DIRECTORY;
    else if (stats.isFile()) type = f.FileSystemEntityType.FILE;
    else if (stats.isSymbolicLink()) type = f.FileSystemEntityType.LINK;

    return new FileStat._internal(type,
      accessed: new DateTime.fromMillisecondsSinceEpoch(stats.atimeMs.truncate()),
      changed: new DateTime.fromMillisecondsSinceEpoch(stats.ctimeMs.truncate()),
      mode: stats.mode,
      modified: new DateTime.fromMillisecondsSinceEpoch(stats.mtimeMs.truncate()),
      size: stats.size
    );
  }

  /// Creates new file stats.
  FileStat._internal(this.type, {
    this.accessed,
    this.changed,
    this.mode = 0,
    this.modified,
    this.size = -1
  });

  /// Calls the operating system's `stat()` function on the specified [path].
  ///
  /// Returns a [Future<FileStat>] object containing the data returned by `stat()`.
  /// If the call fails, completes the future with a [FileStat] object with [type] set to [f.FileSystemEntityType.NOT_FOUND] and the other fields invalid.
  static Future<FileStat> stat(String path) async {
    try { return new FileStat._fromStats(await fn1cb1ToFuture<Stats>(_fs.lstat, path)); }
    on JSError { return new FileStat._internal(f.FileSystemEntityType.NOT_FOUND); }
  }

  /// Synchronously calls the operating system's `stat()` function on the specified [path].
  ///
  /// Returns a [FileStat] object containing the data returned by `stat()`.
  /// If the call fails, returns a [FileStat] object with [type] set to [f.FileSystemEntityType.NOT_FOUND] and the other fields invalid.
  static FileStat statSync(String path) { // ignore: prefer_constructors_over_static_methods
    try { return new FileStat._fromStats(_fs.lstatSync(path)); }
    on JSError { return new FileStat._internal(f.FileSystemEntityType.NOT_FOUND); }
  }

  /// The time of the last access to the data of the file system entity.
  /// On Windows platforms, this may have 1 day granularity, and be out of date by an hour.
  @override
  final DateTime accessed;

  /// The time of the last change to the data or metadata of the file system object.
  /// On Windows platforms, this is instead the file creation time.
  @override
  final DateTime changed;

  /// The time of the last change to the data of the file system object.
  @override
  final DateTime modified;

  /// The mode of the file system entity. Permissions are encoded in the lower 16 bits of this number.
  @override
  final int mode;

  /// The total size, in bytes, of the file system entity.
  @override
  final int size;

  /// The type of the file system entity.
  /// If the call to `stat()` fails, the type of the returned entity is [f.FileSystemEntityType.NOT_FOUND].
  @override
  final f.FileSystemEntityType type;

  /// Returns the [mode] value as a human-readable string, in the format `"rwxrwxrwx"`,
  /// reflecting the user, group, and world permissions to read, write, and execute the file system entity,
  /// with `"-"` replacing the letter for missing permissions.
  ///
  /// Extra permission bits may be represented by prepending `"(suid)"`, `"(guid)"`, and/or `"(sticky)"` to the mode string.
  @override
  String modeString() {
    var codes = const ['---', '--x', '-w-', '-wx', 'r--', 'r-x', 'rw-', 'rwx'];
    var permissions = mode & 0xFFF;

    var result = <String>[];
    if ((permissions & 0x800) != 0) result.add('(suid) ');
    if ((permissions & 0x400) != 0) result.add('(guid) ');
    if ((permissions & 0x200) != 0) result.add('(sticky) ');

    result
      ..add(codes[(permissions >> 6) & 0x7])
      ..add(codes[(permissions >> 3) & 0x7])
      ..add(codes[permissions & 0x7]);

    return result.join();
  }

  /// Returns a string representation of this object.
  @override
  String toString() => 'FileStat("${modeString()}")';
}
